/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.jdbc.EmbeddedRegistry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletContext;

/**
 * Base class for edit processors of custom UIs. Handlers that generate edit or new resource
 * UIs should have an associated EditProcessor implementation. Custom UIs generated by
 * UIEnabledHandlers may have various input controls to get user input for filling up the resource
 * content. EditProcessor implementations should extract these inputs from the request and build
 * the resource content. Once the resource content is built, it should store the resource in the
 * registry.
 */
public abstract class EditProcessor {

    /**
     * Implementations of this method should extract input parameters from edit view and update the
     * resource content.
     *
     * @param path Path of the edited resource.
     * @param editViewKey UI key of the edit UI.
     * @param request HttpServletRequest recieved from the the CustomUIServlet
     * @param response HttpServletResponse to be sent the the CustomUIServlet
     * @throws RegistryException
     */
    public abstract void processEditContent(
            String path,
            String editViewKey,
            HttpServletRequest request, 
            HttpServletResponse response)
            throws RegistryException;

     /**
     * Implementations of this method should extract input parameters from new view and create a
     * new resource with that content.
     *
     * @param path Path of the new resource.
     * @param newViewKey UI key of the new UI.
     * @param request HttpServletRequest recieved from the the CustomUIServlet
     * @param response HttpServletResponse to be sent the the CustomUIServlet
     * @throws RegistryException
     */
    public abstract void processNewContent(
            String path,
            String newViewKey,
            HttpServletRequest request,
            HttpServletResponse response)
            throws RegistryException;

    protected Registry getRegistry(HttpServletRequest request) throws RegistryException {

        Registry registry =
                (Registry) request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

        if (registry == null) {

            // user is not logged in. create a annoymous registry for the user.

            ServletContext context =
                    request.getSession().getServletContext();
            EmbeddedRegistry embeddedRegistry =
                    (EmbeddedRegistry)context.getAttribute(RegistryConstants.REGISTRY);

            UserRegistry userRegistry = embeddedRegistry.getUserRegistry();

            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, userRegistry);

            registry = userRegistry;
        }

        return registry;
    }
}
